@@ -3,21 +3,24 @@ import logging
 
 from collections import defaultdict
 from sortedcontainers import  SortedDict
+from copy import copy
 
 from archinfo.arch_soot import SootMethodDescriptor, SootAddressDescriptor
 
-from .. import register_analysis
+from ...utils.constants import DEFAULT_STATEMENT
 from ...errors import AngrCFGError, SimMemoryError, SimEngineError
 from ...codenode import HookNode, SootBlockNode
+from ...knowledge_plugins.cfg import CFGNode
+from .. import register_analysis
 from .cfg_fast import CFGFast, CFGJob, PendingJobs, FunctionTransitionEdge
-from .cfg_node import CFGNode
 
 l = logging.getLogger(name=__name__)
 
 try:
+    from pysoot.sootir.soot_value import SootLocal
     from pysoot.sootir.soot_statement import IfStmt, InvokeStmt, GotoStmt, AssignStmt
     from pysoot.sootir.soot_expr import SootInterfaceInvokeExpr, SootSpecialInvokeExpr, SootStaticInvokeExpr, \
-        SootVirtualInvokeExpr, SootInvokeExpr, SootDynamicInvokeExpr
+        SootVirtualInvokeExpr, SootInvokeExpr, SootDynamicInvokeExpr  # pylint:disable=unused-import
     PYSOOT_INSTALLED = True
 except ImportError:
     PYSOOT_INSTALLED = False
@@ -25,7 +28,7 @@ except ImportError:
 
 class CFGFastSoot(CFGFast):
 
-    def __init__(self, **kwargs):
+    def __init__(self, support_jni=False, **kwargs):
 
         if not PYSOOT_INSTALLED:
             raise ImportError("Please install PySoot before analyzing Java byte code.")
@@ -34,8 +37,12 @@ class CFGFastSoot(CFGFast):
             raise AngrCFGError('CFGFastSoot only supports analyzing Soot programs.')
 
         self._soot_class_hierarchy = self.project.analyses.SootClassHierarchy()
+        self.support_jni = support_jni
         super(CFGFastSoot, self).__init__(regions=SortedDict({}), **kwargs)
 
+        self._changed_functions = None
+        self._total_methods = None
+
     def _pre_analysis(self):
 
         # Call _initialize_cfg() before self.functions is used.
@@ -47,9 +54,6 @@ class CFGFastSoot(CFGFast):
         self._changed_functions = set()
         self._updated_nonreturning_functions = set()
 
-        self._nodes = {}
-        self._nodes_by_addr = defaultdict(list)
-
         self._function_returns = defaultdict(set)
 
         entry = self.project.entry  # type:SootAddressDescriptor
@@ -118,15 +122,15 @@ class CFGFastSoot(CFGFast):
 
         try:
 
-            if addr in self._nodes:
-                cfg_node = self._nodes[addr]
+            cfg_node = self.model.get_node(addr)
+            if cfg_node is not None:
                 soot_block = cfg_node.soot_block
             else:
                 soot_block = self.project.factory.block(addr).soot
 
                 soot_block_size = self._soot_block_size(soot_block, addr.stmt_idx)
 
-                cfg_node = CFGNode(addr, soot_block_size, self,
+                cfg_node = CFGNode(addr, soot_block_size, self.model,
                                    function_address=current_function_addr, block_id=addr,
                                    soot_block=soot_block
                                    )
@@ -143,11 +147,16 @@ class CFGFastSoot(CFGFast):
 
         return self._soot_get_successors(addr, function_addr, block, cfg_node)
 
-    def _soot_get_successors(self, addr, function_id, block, cfg_node):
+    def _soot_get_successors(self, addr, function_id, block, cfg_node):  # pylint:disable=unused-argument
 
         # soot method
         method = self.project.loader.main_object.get_soot_method(function_id)
 
+        # native method has no soot block
+        if self.support_jni and block is None:
+            successors = self._native_method_successors(addr, method)
+            return successors
+
         block_id = block.idx
 
         if addr.stmt_idx is None:
@@ -176,6 +185,12 @@ class CFGFastSoot(CFGFast):
             elif isinstance(stmt, InvokeStmt):
                 invoke_expr = stmt.invoke_expr
 
+                # add special successors
+                if self.support_jni:
+                    succs = self._special_invoke_successors(stmt, addr, block)
+                    if succs:
+                        successors.extend(succs)
+
                 succs = self._soot_create_invoke_successors(stmt, addr, invoke_expr)
                 if succs:
                     successors.extend(succs)
@@ -197,6 +212,10 @@ class CFGFastSoot(CFGFast):
                 expr = stmt.right_op
 
                 if isinstance(expr, SootInvokeExpr):
+                    succs = self._special_invoke_successors(stmt, addr, block)
+                    if succs:
+                        successors.extend(succs)
+
                     succs = self._soot_create_invoke_successors(stmt, addr, expr)
                     if succs:
                         successors.extend(succs)
@@ -205,7 +224,7 @@ class CFGFastSoot(CFGFast):
 
 
         if has_default_exit:
-            successors.append(('default', addr,
+            successors.append((DEFAULT_STATEMENT, addr,
                                SootAddressDescriptor(function_id, method.block_by_label[next_stmt_id].idx, next_stmt_id),
                                'Ijk_Boring'
                                )
@@ -213,6 +232,69 @@ class CFGFastSoot(CFGFast):
 
         return successors
 
+    def _native_method_successors(self, addr, method):
+        class_name = "nativemethod"
+        # e.g., Java_com_example_nativemedia_NativeMedia_shutdown
+        method_name = "Java_" +  method.class_name.replace('.', '_') + '_' + method.name
+        params = method.params
+        dummy_expr = SootStaticInvokeExpr("void", class_name, method_name, params, {"jni"})
+        dummy_stmt = InvokeStmt(0, 0, dummy_expr)
+        succs_native = self._soot_create_invoke_successors(dummy_stmt, addr, dummy_expr)
+
+        return succs_native
+
+    def _special_invoke_successors(self, stmt, addr, block):
+        invoke_expr = stmt.invoke_expr if isinstance(stmt, InvokeStmt) else stmt.right_op
+        succs = None
+
+        # add <clinit>
+        # many class using jni are loading the library in static method
+        if invoke_expr.method_name == '<init>':
+            clinit_invoke_expr = copy(invoke_expr)
+            clinit_invoke_expr.method_name = '<clinit>'
+            succs = self._soot_create_invoke_successors(stmt, addr, clinit_invoke_expr)
+
+        # convert 'System.loadLibrary' to JNI_OnLoad of library name
+        # format: <libname>.JNI_OnLoad(java.lang.String)
+        elif invoke_expr.class_name == 'System' and invoke_expr.method_name == 'loadLibrary':
+            # Todo: restrictly set condition System.loadlibrary
+            try:
+                native_lib_name = invoke_expr.args[0].value.replace('"', '').replace("'", "")
+                invoke_expr.class_name = native_lib_name
+            except AttributeError:
+                pass
+            invoke_expr.method_name = 'JNI_OnLoad'
+            succs = self._soot_create_invoke_successors(stmt, addr, invoke_expr)
+
+        # add thread.start()
+        # it may occur that block is NoneType when thread call native method.
+        # so only use on support_jni condition
+        # format: <classname>.run()
+        elif invoke_expr.class_name == 'java.lang.Thread' and invoke_expr.method_name == 'start':
+            # Runnable arg case
+            if invoke_expr.base.type == 'java.lang.Thread':
+                thread_class_name = None
+                args = []
+                for before_stmt in block.statements[:block.statements.index(stmt)]:
+                    if isinstance(before_stmt, InvokeStmt):
+                        args.extend(before_stmt.invoke_expr.args)
+
+                # match arg.name == base.name
+                for name in [arg.name for arg in args if isinstance(arg, SootLocal)]:
+                    thread_class_name = name if name == invoke_expr.base.name else None
+
+            # Basic case
+            else:
+                thread_class_name = invoke_expr.base.type
+
+            if thread_class_name is not None:
+                thread_invoke_expr = copy(invoke_expr)
+                thread_invoke_expr.class_name = thread_class_name
+                thread_invoke_expr.method_name = 'run'
+                succs = self._soot_create_invoke_successors(stmt, addr, thread_invoke_expr)
+
+        return succs
+
     def _soot_create_invoke_successors(self, stmt, addr, invoke_expr):
 
         method_class = invoke_expr.class_name
@@ -236,16 +318,13 @@ class CFGFastSoot(CFGFast):
 
         return successors
 
-    def _loc_to_funcloc(self, location):
+    @staticmethod
+    def _loc_to_funcloc(location):
 
         if isinstance(location, SootAddressDescriptor):
             return location.method
         return location
 
-    def _get_plt_stubs(self, functions):
-
-        return set()
-
     def _to_snippet(self, cfg_node=None, addr=None, size=None, thumb=False, jumpkind=None, base_state=None):
 
         assert thumb is False
@@ -281,7 +360,8 @@ class CFGFastSoot(CFGFast):
 
         return SootBlockNode(addr, stmts_count, stmts)
 
-    def _soot_block_size(self, soot_block, start_stmt_idx):
+    @staticmethod
+    def _soot_block_size(soot_block, start_stmt_idx):
 
         if soot_block is None:
             return 0
@@ -369,7 +449,7 @@ class CFGFastSoot(CFGFast):
 
         return entries
 
-    def _create_jobs(self, target, jumpkind, current_function_addr, soot_block, addr, cfg_node, stmt_addr, stmt_idx):
+    def _create_jobs(self, target, jumpkind, current_function_addr, soot_block, addr, cfg_node, stmt_addr, stmt_idx):  # pylint:disable=arguments-differ
 
         """
         Given a node and details of a successor, makes a list of CFGJobs
@@ -460,6 +540,11 @@ class CFGFastSoot(CFGFast):
         :return: None
         """
 
+        # There are some issues in support_jni environment(e.g. _graph_bfs_custom looping)
+        # It handled as passing over for quick fix.
+        if self.support_jni:
+            return
+
         tmp_functions = self.kb.functions.copy()
 
         for function in tmp_functions.values():
@@ -548,7 +633,7 @@ class CFGFastSoot(CFGFast):
             del self.kb.functions[addr]
 
         # Update CFGNode.function_address
-        for node in self._nodes.values():
+        for node in self.model.nodes():
             if node.addr in blockaddr_to_function:
                 node.function_address = blockaddr_to_function[node.addr].addr
 
